@@ -6,32 +6,30 @@
 #![allow(unsafe_code)]
 #![allow(clippy::undocumented_unsafe_blocks)]
 
-use super::super::c;
-use super::super::conv::{
-    by_mut, by_ref, c_int, c_uint, ret, ret_owned_fd, ret_usize, size_of, slice, slice_mut,
-    socklen_t, zero,
-};
 use super::msghdr::{
     with_noaddr_msghdr, with_recv_msghdr, with_unix_msghdr, with_v4_msghdr, with_v6_msghdr,
 };
 use super::read_sockaddr::{initialize_family_to_unspec, maybe_read_sockaddr_os, read_sockaddr_os};
 use super::send_recv::{RecvFlags, SendFlags};
-use super::types::{AddressFamily, Protocol, Shutdown, SocketFlags, SocketType};
 use super::write_sockaddr::{encode_sockaddr_v4, encode_sockaddr_v6};
+use crate::backend::c;
+use crate::backend::conv::{
+    by_mut, by_ref, c_int, c_uint, ret, ret_owned_fd, ret_usize, size_of, slice, slice_mut,
+    socklen_t, zero,
+};
 use crate::fd::{BorrowedFd, OwnedFd};
 use crate::io::{self, IoSlice, IoSliceMut};
 use crate::net::{
-    RecvAncillaryBuffer, RecvMsgReturn, SendAncillaryBuffer, SocketAddrAny, SocketAddrUnix,
-    SocketAddrV4, SocketAddrV6,
+    AddressFamily, Protocol, RecvAncillaryBuffer, RecvMsgReturn, SendAncillaryBuffer, Shutdown,
+    SocketAddrAny, SocketAddrUnix, SocketAddrV4, SocketAddrV6, SocketFlags, SocketType,
 };
 use c::{sockaddr, sockaddr_in, sockaddr_in6, socklen_t};
-use core::convert::TryInto;
 use core::mem::MaybeUninit;
 #[cfg(target_arch = "x86")]
 use {
-    super::super::conv::{slice_just_addr, x86_sys},
-    super::super::reg::{ArgReg, SocketArg},
-    linux_raw_sys::general::{
+    crate::backend::conv::{slice_just_addr, x86_sys},
+    crate::backend::reg::{ArgReg, SocketArg},
+    linux_raw_sys::net::{
         SYS_ACCEPT, SYS_ACCEPT4, SYS_BIND, SYS_CONNECT, SYS_GETPEERNAME, SYS_GETSOCKNAME,
         SYS_GETSOCKOPT, SYS_LISTEN, SYS_RECV, SYS_RECVFROM, SYS_RECVMSG, SYS_SEND, SYS_SENDMSG,
         SYS_SENDTO, SYS_SETSOCKOPT, SYS_SHUTDOWN, SYS_SOCKET, SYS_SOCKETPAIR,
@@ -42,7 +40,7 @@ use {
 pub(crate) fn socket(
     family: AddressFamily,
     type_: SocketType,
-    protocol: Protocol,
+    protocol: Option<Protocol>,
 ) -> io::Result<OwnedFd> {
     #[cfg(not(target_arch = "x86"))]
     unsafe {
@@ -53,7 +51,7 @@ pub(crate) fn socket(
         ret_owned_fd(syscall_readonly!(
             __NR_socketcall,
             x86_sys(SYS_SOCKET),
-            slice_just_addr::<ArgReg<SocketArg>, _>(&[
+            slice_just_addr::<ArgReg<'_, SocketArg>, _>(&[
                 family.into(),
                 type_.into(),
                 protocol.into(),
@@ -67,7 +65,7 @@ pub(crate) fn socket_with(
     family: AddressFamily,
     type_: SocketType,
     flags: SocketFlags,
-    protocol: Protocol,
+    protocol: Option<Protocol>,
 ) -> io::Result<OwnedFd> {
     #[cfg(not(target_arch = "x86"))]
     unsafe {
@@ -83,7 +81,7 @@ pub(crate) fn socket_with(
         ret_owned_fd(syscall_readonly!(
             __NR_socketcall,
             x86_sys(SYS_SOCKET),
-            slice_just_addr::<ArgReg<SocketArg>, _>(&[
+            slice_just_addr::<ArgReg<'_, SocketArg>, _>(&[
                 family.into(),
                 (type_, flags).into(),
                 protocol.into(),
@@ -97,7 +95,7 @@ pub(crate) fn socketpair(
     family: AddressFamily,
     type_: SocketType,
     flags: SocketFlags,
-    protocol: Protocol,
+    protocol: Option<Protocol>,
 ) -> io::Result<(OwnedFd, OwnedFd)> {
     #[cfg(not(target_arch = "x86"))]
     unsafe {
@@ -118,7 +116,7 @@ pub(crate) fn socketpair(
         ret(syscall!(
             __NR_socketcall,
             x86_sys(SYS_SOCKETPAIR),
-            slice_just_addr::<ArgReg<SocketArg>, _>(&[
+            slice_just_addr::<ArgReg<'_, SocketArg>, _>(&[
                 family.into(),
                 (type_, flags).into(),
                 protocol.into(),
@@ -142,7 +140,7 @@ pub(crate) fn accept(fd: BorrowedFd<'_>) -> io::Result<OwnedFd> {
         let fd = ret_owned_fd(syscall_readonly!(
             __NR_socketcall,
             x86_sys(SYS_ACCEPT),
-            slice_just_addr::<ArgReg<SocketArg>, _>(&[fd.into(), zero(), zero()])
+            slice_just_addr::<ArgReg<'_, SocketArg>, _>(&[fd.into(), zero(), zero()])
         ))?;
         Ok(fd)
     }
@@ -160,7 +158,7 @@ pub(crate) fn accept_with(fd: BorrowedFd<'_>, flags: SocketFlags) -> io::Result<
         let fd = ret_owned_fd(syscall_readonly!(
             __NR_socketcall,
             x86_sys(SYS_ACCEPT4),
-            slice_just_addr::<ArgReg<SocketArg>, _>(&[fd.into(), zero(), zero(), flags.into()])
+            slice_just_addr::<ArgReg<'_, SocketArg>, _>(&[fd.into(), zero(), zero(), flags.into()])
         ))?;
         Ok(fd)
     }
@@ -190,7 +188,7 @@ pub(crate) fn acceptfrom(fd: BorrowedFd<'_>) -> io::Result<(OwnedFd, Option<Sock
         let fd = ret_owned_fd(syscall!(
             __NR_socketcall,
             x86_sys(SYS_ACCEPT),
-            slice_just_addr::<ArgReg<SocketArg>, _>(&[
+            slice_just_addr::<ArgReg<'_, SocketArg>, _>(&[
                 fd.into(),
                 (&mut storage).into(),
                 by_mut(&mut addrlen),
@@ -231,7 +229,7 @@ pub(crate) fn acceptfrom_with(
         let fd = ret_owned_fd(syscall!(
             __NR_socketcall,
             x86_sys(SYS_ACCEPT4),
-            slice_just_addr::<ArgReg<SocketArg>, _>(&[
+            slice_just_addr::<ArgReg<'_, SocketArg>, _>(&[
                 fd.into(),
                 (&mut storage).into(),
                 by_mut(&mut addrlen),
@@ -256,24 +254,18 @@ pub(crate) fn recvmsg(
 
     with_recv_msghdr(&mut storage, iov, control, |msghdr| {
         #[cfg(not(target_arch = "x86"))]
-        let result = unsafe {
-            ret_usize(syscall!(
-                __NR_recvmsg,
-                sockfd,
-                by_mut(msghdr),
-                c_uint(msg_flags.bits())
-            ))
-        };
+        let result =
+            unsafe { ret_usize(syscall!(__NR_recvmsg, sockfd, by_mut(msghdr), msg_flags)) };
 
         #[cfg(target_arch = "x86")]
         let result = unsafe {
             ret_usize(syscall!(
                 __NR_socketcall,
                 x86_sys(SYS_RECVMSG),
-                slice_just_addr::<ArgReg<SocketArg>, _>(&[
+                slice_just_addr::<ArgReg<'_, SocketArg>, _>(&[
                     sockfd.into(),
                     by_mut(msghdr),
-                    c_uint(msg_flags.bits()).into(),
+                    msg_flags.into(),
                 ])
             ))
         };
@@ -286,14 +278,14 @@ pub(crate) fn recvmsg(
             RecvMsgReturn {
                 bytes,
                 address: addr,
-                flags: RecvFlags::from_bits_truncate(msghdr.msg_flags),
+                flags: RecvFlags::from_bits_retain(msghdr.msg_flags),
             }
         })
     })
 }
 
 #[inline]
-pub(crate) fn sendmsg_noaddr(
+pub(crate) fn sendmsg(
     sockfd: BorrowedFd<'_>,
     iov: &[IoSlice<'_>],
     control: &mut SendAncillaryBuffer<'_, '_, '_>,
@@ -301,24 +293,18 @@ pub(crate) fn sendmsg_noaddr(
 ) -> io::Result<usize> {
     with_noaddr_msghdr(iov, control, |msghdr| {
         #[cfg(not(target_arch = "x86"))]
-        let result = unsafe {
-            ret_usize(syscall!(
-                __NR_sendmsg,
-                sockfd,
-                by_ref(&msghdr),
-                c_uint(msg_flags.bits())
-            ))
-        };
+        let result =
+            unsafe { ret_usize(syscall!(__NR_sendmsg, sockfd, by_ref(&msghdr), msg_flags)) };
 
         #[cfg(target_arch = "x86")]
         let result = unsafe {
             ret_usize(syscall!(
                 __NR_socketcall,
                 x86_sys(SYS_SENDMSG),
-                slice_just_addr::<ArgReg<SocketArg>, _>(&[
+                slice_just_addr::<ArgReg<'_, SocketArg>, _>(&[
                     sockfd.into(),
                     by_ref(&msghdr),
-                    c_uint(msg_flags.bits())
+                    msg_flags.into()
                 ])
             ))
         };
@@ -337,24 +323,18 @@ pub(crate) fn sendmsg_v4(
 ) -> io::Result<usize> {
     with_v4_msghdr(addr, iov, control, |msghdr| {
         #[cfg(not(target_arch = "x86"))]
-        let result = unsafe {
-            ret_usize(syscall!(
-                __NR_sendmsg,
-                sockfd,
-                by_ref(&msghdr),
-                c_uint(msg_flags.bits())
-            ))
-        };
+        let result =
+            unsafe { ret_usize(syscall!(__NR_sendmsg, sockfd, by_ref(&msghdr), msg_flags)) };
 
         #[cfg(target_arch = "x86")]
         let result = unsafe {
             ret_usize(syscall!(
                 __NR_socketcall,
                 x86_sys(SYS_SENDMSG),
-                slice_just_addr::<ArgReg<SocketArg>, _>(&[
+                slice_just_addr::<ArgReg<'_, SocketArg>, _>(&[
                     sockfd.into(),
                     by_ref(&msghdr),
-                    c_uint(msg_flags.bits())
+                    msg_flags.into(),
                 ])
             ))
         };
@@ -373,24 +353,18 @@ pub(crate) fn sendmsg_v6(
 ) -> io::Result<usize> {
     with_v6_msghdr(addr, iov, control, |msghdr| {
         #[cfg(not(target_arch = "x86"))]
-        let result = unsafe {
-            ret_usize(syscall!(
-                __NR_sendmsg,
-                sockfd,
-                by_ref(&msghdr),
-                c_uint(msg_flags.bits())
-            ))
-        };
+        let result =
+            unsafe { ret_usize(syscall!(__NR_sendmsg, sockfd, by_ref(&msghdr), msg_flags)) };
 
         #[cfg(target_arch = "x86")]
         let result = unsafe {
             ret_usize(syscall!(
                 __NR_socketcall,
                 x86_sys(SYS_SENDMSG),
-                slice_just_addr::<ArgReg<SocketArg>, _>(&[
+                slice_just_addr::<ArgReg<'_, SocketArg>, _>(&[
                     sockfd.into(),
                     by_ref(&msghdr),
-                    c_uint(msg_flags.bits())
+                    msg_flags.into()
                 ])
             ))
         };
@@ -409,24 +383,18 @@ pub(crate) fn sendmsg_unix(
 ) -> io::Result<usize> {
     with_unix_msghdr(addr, iov, control, |msghdr| {
         #[cfg(not(target_arch = "x86"))]
-        let result = unsafe {
-            ret_usize(syscall!(
-                __NR_sendmsg,
-                sockfd,
-                by_ref(&msghdr),
-                c_uint(msg_flags.bits())
-            ))
-        };
+        let result =
+            unsafe { ret_usize(syscall!(__NR_sendmsg, sockfd, by_ref(&msghdr), msg_flags)) };
 
         #[cfg(target_arch = "x86")]
         let result = unsafe {
             ret_usize(syscall!(
                 __NR_socketcall,
                 x86_sys(SYS_SENDMSG),
-                slice_just_addr::<ArgReg<SocketArg>, _>(&[
+                slice_just_addr::<ArgReg<'_, SocketArg>, _>(&[
                     sockfd.into(),
                     by_ref(&msghdr),
-                    c_uint(msg_flags.bits())
+                    msg_flags.into()
                 ])
             ))
         };
@@ -450,7 +418,7 @@ pub(crate) fn shutdown(fd: BorrowedFd<'_>, how: Shutdown) -> io::Result<()> {
         ret(syscall_readonly!(
             __NR_socketcall,
             x86_sys(SYS_SHUTDOWN),
-            slice_just_addr::<ArgReg<SocketArg>, _>(&[fd.into(), c_uint(how as c::c_uint)])
+            slice_just_addr::<ArgReg<'_, SocketArg>, _>(&[fd.into(), c_uint(how as c::c_uint)])
         ))
     }
 }
@@ -462,6 +430,7 @@ pub(crate) fn send(fd: BorrowedFd<'_>, buf: &[u8], flags: SendFlags) -> io::Resu
     #[cfg(not(any(
         target_arch = "aarch64",
         target_arch = "mips64",
+        target_arch = "mips64r6",
         target_arch = "riscv64",
         target_arch = "x86",
         target_arch = "x86_64",
@@ -472,6 +441,7 @@ pub(crate) fn send(fd: BorrowedFd<'_>, buf: &[u8], flags: SendFlags) -> io::Resu
     #[cfg(any(
         target_arch = "aarch64",
         target_arch = "mips64",
+        target_arch = "mips64r6",
         target_arch = "riscv64",
         target_arch = "x86_64",
     ))]
@@ -491,7 +461,12 @@ pub(crate) fn send(fd: BorrowedFd<'_>, buf: &[u8], flags: SendFlags) -> io::Resu
         ret_usize(syscall_readonly!(
             __NR_socketcall,
             x86_sys(SYS_SEND),
-            slice_just_addr::<ArgReg<SocketArg>, _>(&[fd.into(), buf_addr, buf_len, flags.into()])
+            slice_just_addr::<ArgReg<'_, SocketArg>, _>(&[
+                fd.into(),
+                buf_addr,
+                buf_len,
+                flags.into()
+            ])
         ))
     }
 }
@@ -522,7 +497,7 @@ pub(crate) fn sendto_v4(
         ret_usize(syscall_readonly!(
             __NR_socketcall,
             x86_sys(SYS_SENDTO),
-            slice_just_addr::<ArgReg<SocketArg>, _>(&[
+            slice_just_addr::<ArgReg<'_, SocketArg>, _>(&[
                 fd.into(),
                 buf_addr,
                 buf_len,
@@ -560,7 +535,7 @@ pub(crate) fn sendto_v6(
         ret_usize(syscall_readonly!(
             __NR_socketcall,
             x86_sys(SYS_SENDTO),
-            slice_just_addr::<ArgReg<SocketArg>, _>(&[
+            slice_just_addr::<ArgReg<'_, SocketArg>, _>(&[
                 fd.into(),
                 buf_addr,
                 buf_len,
@@ -598,7 +573,7 @@ pub(crate) fn sendto_unix(
         ret_usize(syscall_readonly!(
             __NR_socketcall,
             x86_sys(SYS_SENDTO),
-            slice_just_addr::<ArgReg<SocketArg>, _>(&[
+            slice_just_addr::<ArgReg<'_, SocketArg>, _>(&[
                 fd.into(),
                 buf_addr,
                 buf_len,
@@ -617,6 +592,7 @@ pub(crate) fn recv(fd: BorrowedFd<'_>, buf: &mut [u8], flags: RecvFlags) -> io::
     #[cfg(not(any(
         target_arch = "aarch64",
         target_arch = "mips64",
+        target_arch = "mips64r6",
         target_arch = "riscv64",
         target_arch = "x86",
         target_arch = "x86_64",
@@ -627,6 +603,7 @@ pub(crate) fn recv(fd: BorrowedFd<'_>, buf: &mut [u8], flags: RecvFlags) -> io::
     #[cfg(any(
         target_arch = "aarch64",
         target_arch = "mips64",
+        target_arch = "mips64r6",
         target_arch = "riscv64",
         target_arch = "x86_64",
     ))]
@@ -646,7 +623,7 @@ pub(crate) fn recv(fd: BorrowedFd<'_>, buf: &mut [u8], flags: RecvFlags) -> io::
         ret_usize(syscall!(
             __NR_socketcall,
             x86_sys(SYS_RECV),
-            slice_just_addr::<ArgReg<SocketArg>, _>(&[
+            slice_just_addr::<ArgReg<'_, SocketArg>, _>(&[
                 fd.into(),
                 buf_addr_mut,
                 buf_len,
@@ -687,7 +664,7 @@ pub(crate) fn recvfrom(
         let nread = ret_usize(syscall!(
             __NR_socketcall,
             x86_sys(SYS_RECVFROM),
-            slice_just_addr::<ArgReg<SocketArg>, _>(&[
+            slice_just_addr::<ArgReg<'_, SocketArg>, _>(&[
                 fd.into(),
                 buf_addr_mut,
                 buf_len,
@@ -728,7 +705,7 @@ pub(crate) fn getpeername(fd: BorrowedFd<'_>) -> io::Result<Option<SocketAddrAny
         ret(syscall!(
             __NR_socketcall,
             x86_sys(SYS_GETPEERNAME),
-            slice_just_addr::<ArgReg<SocketArg>, _>(&[
+            slice_just_addr::<ArgReg<'_, SocketArg>, _>(&[
                 fd.into(),
                 (&mut storage).into(),
                 by_mut(&mut addrlen),
@@ -765,7 +742,7 @@ pub(crate) fn getsockname(fd: BorrowedFd<'_>) -> io::Result<SocketAddrAny> {
         ret(syscall!(
             __NR_socketcall,
             x86_sys(SYS_GETSOCKNAME),
-            slice_just_addr::<ArgReg<SocketArg>, _>(&[
+            slice_just_addr::<ArgReg<'_, SocketArg>, _>(&[
                 fd.into(),
                 (&mut storage).into(),
                 by_mut(&mut addrlen),
@@ -794,7 +771,7 @@ pub(crate) fn bind_v4(fd: BorrowedFd<'_>, addr: &SocketAddrV4) -> io::Result<()>
         ret(syscall_readonly!(
             __NR_socketcall,
             x86_sys(SYS_BIND),
-            slice_just_addr::<ArgReg<SocketArg>, _>(&[
+            slice_just_addr::<ArgReg<'_, SocketArg>, _>(&[
                 fd.into(),
                 by_ref(&encode_sockaddr_v4(addr)),
                 size_of::<sockaddr_in, _>(),
@@ -819,7 +796,7 @@ pub(crate) fn bind_v6(fd: BorrowedFd<'_>, addr: &SocketAddrV6) -> io::Result<()>
         ret(syscall_readonly!(
             __NR_socketcall,
             x86_sys(SYS_BIND),
-            slice_just_addr::<ArgReg<SocketArg>, _>(&[
+            slice_just_addr::<ArgReg<'_, SocketArg>, _>(&[
                 fd.into(),
                 by_ref(&encode_sockaddr_v6(addr)),
                 size_of::<sockaddr_in6, _>(),
@@ -844,7 +821,7 @@ pub(crate) fn bind_unix(fd: BorrowedFd<'_>, addr: &SocketAddrUnix) -> io::Result
         ret(syscall_readonly!(
             __NR_socketcall,
             x86_sys(SYS_BIND),
-            slice_just_addr::<ArgReg<SocketArg>, _>(&[
+            slice_just_addr::<ArgReg<'_, SocketArg>, _>(&[
                 fd.into(),
                 by_ref(&addr.unix),
                 socklen_t(addr.addr_len()),
@@ -869,7 +846,7 @@ pub(crate) fn connect_v4(fd: BorrowedFd<'_>, addr: &SocketAddrV4) -> io::Result<
         ret(syscall_readonly!(
             __NR_socketcall,
             x86_sys(SYS_CONNECT),
-            slice_just_addr::<ArgReg<SocketArg>, _>(&[
+            slice_just_addr::<ArgReg<'_, SocketArg>, _>(&[
                 fd.into(),
                 by_ref(&encode_sockaddr_v4(addr)),
                 size_of::<sockaddr_in, _>(),
@@ -894,7 +871,7 @@ pub(crate) fn connect_v6(fd: BorrowedFd<'_>, addr: &SocketAddrV6) -> io::Result<
         ret(syscall_readonly!(
             __NR_socketcall,
             x86_sys(SYS_CONNECT),
-            slice_just_addr::<ArgReg<SocketArg>, _>(&[
+            slice_just_addr::<ArgReg<'_, SocketArg>, _>(&[
                 fd.into(),
                 by_ref(&encode_sockaddr_v6(addr)),
                 size_of::<sockaddr_in6, _>(),
@@ -919,7 +896,7 @@ pub(crate) fn connect_unix(fd: BorrowedFd<'_>, addr: &SocketAddrUnix) -> io::Res
         ret(syscall_readonly!(
             __NR_socketcall,
             x86_sys(SYS_CONNECT),
-            slice_just_addr::<ArgReg<SocketArg>, _>(&[
+            slice_just_addr::<ArgReg<'_, SocketArg>, _>(&[
                 fd.into(),
                 by_ref(&addr.unix),
                 socklen_t(addr.addr_len()),
@@ -939,7 +916,7 @@ pub(crate) fn listen(fd: BorrowedFd<'_>, backlog: c::c_int) -> io::Result<()> {
         ret(syscall_readonly!(
             __NR_socketcall,
             x86_sys(SYS_LISTEN),
-            slice_just_addr::<ArgReg<SocketArg>, _>(&[fd.into(), c_int(backlog)])
+            slice_just_addr::<ArgReg<'_, SocketArg>, _>(&[fd.into(), c_int(backlog)])
         ))
     }
 }
@@ -948,15 +925,11 @@ pub(crate) mod sockopt {
     use super::{c, BorrowedFd};
     use crate::io;
     use crate::net::sockopt::Timeout;
-    use crate::net::{Ipv4Addr, Ipv6Addr, SocketType};
+    use crate::net::{AddressFamily, Ipv4Addr, Ipv6Addr, SocketType};
     use c::{SO_RCVTIMEO_NEW, SO_RCVTIMEO_OLD, SO_SNDTIMEO_NEW, SO_SNDTIMEO_OLD};
-    use core::convert::TryInto;
     use core::time::Duration;
     use linux_raw_sys::general::{__kernel_timespec, timeval};
 
-    // TODO: With Rust 1.53 we can use `Duration::ZERO` instead.
-    const DURATION_ZERO: Duration = Duration::from_secs(0);
-
     #[inline]
     fn getsockopt<T: Copy>(fd: BorrowedFd<'_>, level: u32, optname: u32) -> io::Result<T> {
         use super::*;
@@ -992,7 +965,7 @@ pub(crate) mod sockopt {
             ret(syscall!(
                 __NR_socketcall,
                 x86_sys(SYS_GETSOCKOPT),
-                slice_just_addr::<ArgReg<SocketArg>, _>(&[
+                slice_just_addr::<ArgReg<'_, SocketArg>, _>(&[
                     fd.into(),
                     c_uint(level),
                     c_uint(optname),
@@ -1040,7 +1013,7 @@ pub(crate) mod sockopt {
             ret(syscall_readonly!(
                 __NR_socketcall,
                 x86_sys(SYS_SETSOCKOPT),
-                slice_just_addr::<ArgReg<SocketArg>, _>(&[
+                slice_just_addr::<ArgReg<'_, SocketArg>, _>(&[
                     fd.into(),
                     c_uint(level),
                     c_uint(optname),
@@ -1106,12 +1079,7 @@ pub(crate) mod sockopt {
     #[inline]
     pub(crate) fn get_socket_linger(fd: BorrowedFd<'_>) -> io::Result<Option<Duration>> {
         let linger: c::linger = getsockopt(fd, c::SOL_SOCKET as _, c::SO_LINGER)?;
-        // TODO: With Rust 1.50, this could use `.then`.
-        Ok(if linger.l_onoff != 0 {
-            Some(Duration::from_secs(linger.l_linger as u64))
-        } else {
-            None
-        })
+        Ok((linger.l_onoff != 0).then(|| Duration::from_secs(linger.l_linger as u64)))
     }
 
     #[inline]
@@ -1216,7 +1184,7 @@ pub(crate) mod sockopt {
     fn duration_to_linux(timeout: Option<Duration>) -> io::Result<__kernel_timespec> {
         Ok(match timeout {
             Some(timeout) => {
-                if timeout == DURATION_ZERO {
+                if timeout == Duration::ZERO {
                     return Err(io::Errno::INVAL);
                 }
                 let mut timeout = __kernel_timespec {
@@ -1239,7 +1207,7 @@ pub(crate) mod sockopt {
     fn duration_to_linux_old(timeout: Option<Duration>) -> io::Result<timeval> {
         Ok(match timeout {
             Some(timeout) => {
-                if timeout == DURATION_ZERO {
+                if timeout == Duration::ZERO {
                     return Err(io::Errno::INVAL);
                 }
 
@@ -1308,6 +1276,14 @@ pub(crate) mod sockopt {
         getsockopt(fd, c::SOL_SOCKET as _, c::SO_SNDBUF).map(|size: u32| size as usize)
     }
 
+    #[inline]
+    pub(crate) fn get_socket_domain(fd: BorrowedFd<'_>) -> io::Result<AddressFamily> {
+        let domain: c::c_int = getsockopt(fd, c::SOL_SOCKET as _, c::SO_DOMAIN)?;
+        Ok(AddressFamily(
+            domain.try_into().map_err(|_| io::Errno::OPNOTSUPP)?,
+        ))
+    }
+
     #[inline]
     pub(crate) fn set_ip_ttl(fd: BorrowedFd<'_>, ttl: u32) -> io::Result<()> {
         setsockopt(fd, c::IPPROTO_IP as _, c::IP_TTL, ttl)
@@ -1440,7 +1416,7 @@ pub(crate) mod sockopt {
     #[inline]
     pub(crate) fn set_ipv6_unicast_hops(fd: BorrowedFd<'_>, hops: Option<u8>) -> io::Result<()> {
         let hops = match hops {
-            Some(hops) => hops as c::c_int,
+            Some(hops) => hops.into(),
             None => -1,
         };
         setsockopt(fd, c::IPPROTO_IPV6 as _, c::IPV6_UNICAST_HOPS, hops)
@@ -1482,7 +1458,7 @@ pub(crate) mod sockopt {
     #[inline]
     fn to_ipv6mr_multiaddr(multiaddr: &Ipv6Addr) -> c::in6_addr {
         c::in6_addr {
-            in6_u: linux_raw_sys::general::in6_addr__bindgen_ty_1 {
+            in6_u: linux_raw_sys::net::in6_addr__bindgen_ty_1 {
                 u6_addr8: multiaddr.octets(),
             },
         }
